@@ -4,6 +4,7 @@ from traceback import format_exception
 from collections.abc import Coroutine, Generator
 from inspect import iscoroutinefunction, isgeneratorfunction
 import contextvars
+import outcome
 import pytest
 import trio
 from trio.testing import MockClock, trio_test
@@ -132,11 +133,16 @@ class TrioTestContext:
     def __init__(self):
         self.crashed = False
         self.test_cancel_scope = None
+        self.fixtures_with_errors = set()
+        self.fixtures_with_cancel = set()
         self.error_list = []
 
-    def crash(self, exc):
-        if exc is not None:
+    def crash(self, fixture, exc):
+        if exc is None:
+            self.fixtures_with_cancel.add(fixture)
+        else:
             self.error_list.append(exc)
+            self.fixtures_with_errors.add(fixture)
         self.crashed = True
         if self.test_cancel_scope is not None:
             self.test_cancel_scope.cancel()
@@ -192,7 +198,7 @@ class TrioFixture:
                 finally:
                     nursery_fixture.cancel_scope.cancel()
         except BaseException as exc:
-            test_ctx.crash(exc)
+            test_ctx.crash(self, exc)
         finally:
             self.setup_done.set()
             self._teardown_done.set()
@@ -202,7 +208,7 @@ class TrioFixture:
 
         # This is a gross hack. I guess Trio should provide a context=
         # argument to start_soon/start?
-        task = trio.hazmat.current_task()
+        task = trio.lowlevel.current_task()
         assert canary not in task.context
         task.context = contextvars_ctx
         # Force a yield so we pick up the new context
@@ -278,12 +284,14 @@ class TrioFixture:
             # code will get it again if it matters), and then use a shield to
             # keep waiting for the teardown to finish without having to worry
             # about cancellation.
+            yield_outcome = outcome.Value(None)
             try:
                 for event in self.user_done_events:
                     await event.wait()
             except BaseException as exc:
                 assert isinstance(exc, trio.Cancelled)
-                test_ctx.crash(None)
+                yield_outcome = outcome.Error(exc)
+                test_ctx.crash(self, None)
                 with trio.CancelScope(shield=True):
                     for event in self.user_done_events:
                         await event.wait()
@@ -291,14 +299,14 @@ class TrioFixture:
             # Do our teardown
             if isasyncgen(func_value):
                 try:
-                    await func_value.asend(None)
+                    await yield_outcome.asend(func_value)
                 except StopAsyncIteration:
                     pass
                 else:
                     raise RuntimeError("too many yields in fixture")
             elif isinstance(func_value, Generator):
                 try:
-                    func_value.send(None)
+                    yield_outcome.send(func_value)
                 except StopIteration:
                     pass
                 else:
@@ -339,6 +347,18 @@ def _trio_test_runner_factory(item, testfunc=None):
                     fixture.run, test_ctx, contextvars_ctx, name=fixture.name
                 )
 
+        silent_cancellers = (
+            test_ctx.fixtures_with_cancel - test_ctx.fixtures_with_errors
+        )
+        if silent_cancellers:
+            for fixture in silent_cancellers:
+                test_ctx.error_list.append(
+                    RuntimeError(
+                        "{} cancelled the test but didn't "
+                        "raise an error".format(fixture.name)
+                    )
+                )
+
         if test_ctx.error_list:
             raise trio.MultiError(test_ctx.error_list)
 
